@@ -6,6 +6,7 @@ from .spod_base        import SPOD_base
 from .spod_low_storage import SPOD_low_storage
 from .spod_low_ram     import SPOD_low_ram
 from .spod_streaming   import SPOD_streaming
+
 # from pyspod import SPOD_low_storage, SPOD_low_ram, SPOD_streaming
 
 import os
@@ -19,11 +20,11 @@ for P in PACKAGE_PARENTS:
 
 __project__ = 'PySPOD'
 __title__ = "pyspod"
-__author__ = "Gianmarco Mengaldo, Romit Maulik"
-__email__ = 'gianmarco.mengaldo@gmail.com, rmaulik@anl.gov'
-__copyright__ = "Copyright 2020-2021 PySPOD authors and contributors"
+__author__ = "Gianmarco Mengaldo, Romit Maulik, Andrea Lario"
+__email__ = 'mpegim@nus.edu.sg, rmaulik@anl.gov, alario@sissa.it'
+__copyright__ = "Copyright 2020-2022 PySPOD authors and contributors"
 __maintainer__ = __author__
 __status__ = "Stable"
 __license__ = "MIT"
-__version__ = "0.5.0"
+__version__ = "1.0.0"
 __url__ = "https://github.com/mengaldo/PySPOD"
